# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal, TypeAlias

__all__ = ["BroadcastContactStatus"]

BroadcastContactStatus: TypeAlias = Literal["pending", "queued", "sending", "delivered", "failed", "skipped"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Optional
from typing_extensions import Literal, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["ContactUpdateParams"]


class ContactUpdateParams(TypedDict, total=False):
    default_channel: Annotated[
        Optional[Literal["sms", "whatsapp", "telegram", "email", "instagram", "voice"]],
        PropertyInfo(alias="defaultChannel"),
    ]
    """Preferred channel for this contact. Set to null to clear."""

    metadata: Dict[str, str]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from pydantic import Field as FieldInfo

from .._models import BaseModel
from .requirement_field_type import RequirementFieldType
from .requirement_acceptance_criteria import RequirementAcceptanceCriteria

__all__ = ["RequirementType"]


class RequirementType(BaseModel):
    """A specific requirement type within a requirement group."""

    id: str

    description: str

    name: str

    type: RequirementFieldType
    """Type of requirement field."""

    acceptance_criteria: Optional[RequirementAcceptanceCriteria] = FieldInfo(alias="acceptanceCriteria", default=None)
    """Acceptance criteria for a requirement."""

    example: Optional[str] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from ..._models import BaseModel

__all__ = ["ContactAddResponse", "Error"]


class Error(BaseModel):
    reason: Optional[str] = None

    recipient: Optional[str] = None


class ContactAddResponse(BaseModel):
    added: int
    """Number of contacts successfully added."""

    duplicates: int
    """Number of duplicate contacts skipped."""

    invalid: int
    """Number of invalid contacts rejected."""

    errors: Optional[List[Error]] = None
    """Details about invalid contacts."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict
from typing_extensions import Required, Annotated, TypedDict

from ...._utils import PropertyInfo

__all__ = ["ToolTestParams"]


class ToolTestParams(TypedDict, total=False):
    sender_id: Required[Annotated[str, PropertyInfo(alias="senderId")]]

    test_params: Required[Annotated[Dict[str, object], PropertyInfo(alias="testParams")]]
    """Parameters to pass to the tool for testing."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .contacts import (
    ContactsResource,
    AsyncContactsResource,
    ContactsResourceWithRawResponse,
    AsyncContactsResourceWithRawResponse,
    ContactsResourceWithStreamingResponse,
    AsyncContactsResourceWithStreamingResponse,
)
from .broadcasts import (
    BroadcastsResource,
    AsyncBroadcastsResource,
    BroadcastsResourceWithRawResponse,
    AsyncBroadcastsResourceWithRawResponse,
    BroadcastsResourceWithStreamingResponse,
    AsyncBroadcastsResourceWithStreamingResponse,
)

__all__ = [
    "ContactsResource",
    "AsyncContactsResource",
    "ContactsResourceWithRawResponse",
    "AsyncContactsResourceWithRawResponse",
    "ContactsResourceWithStreamingResponse",
    "AsyncContactsResourceWithStreamingResponse",
    "BroadcastsResource",
    "AsyncBroadcastsResource",
    "BroadcastsResourceWithRawResponse",
    "AsyncBroadcastsResourceWithRawResponse",
    "BroadcastsResourceWithStreamingResponse",
    "AsyncBroadcastsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Union
from datetime import datetime

import httpx

from ...types import (
    BroadcastStatus,
    BroadcastChannel,
    BroadcastMessageType,
    broadcast_list_params,
    broadcast_send_params,
    broadcast_create_params,
    broadcast_update_params,
    broadcast_reschedule_params,
)
from ..._types import Body, Omit, Query, Headers, NoneType, NotGiven, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from .contacts import (
    ContactsResource,
    AsyncContactsResource,
    ContactsResourceWithRawResponse,
    AsyncContactsResourceWithRawResponse,
    ContactsResourceWithStreamingResponse,
    AsyncContactsResourceWithStreamingResponse,
)
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...pagination import SyncCursor, AsyncCursor
from ..._base_client import AsyncPaginator, make_request_options
from ...types.broadcast import Broadcast
from ...types.broadcast_status import BroadcastStatus
from ...types.broadcast_channel import BroadcastChannel
from ...types.broadcast_progress import BroadcastProgress
from ...types.broadcast_message_type import BroadcastMessageType
from ...types.broadcast_content_param import BroadcastContentParam
from ...types.broadcast_send_response import BroadcastSendResponse
from ...types.broadcast_cancel_response import BroadcastCancelResponse
from ...types.broadcast_create_response import BroadcastCreateResponse
from ...types.broadcast_update_response import BroadcastUpdateResponse
from ...types.broadcast_retrieve_response import BroadcastRetrieveResponse
from ...types.broadcast_reschedule_response import BroadcastRescheduleResponse

__all__ = ["BroadcastsResource", "AsyncBroadcastsResource"]


class BroadcastsResource(SyncAPIResource):
    @cached_property
    def contacts(self) -> ContactsResource:
        return ContactsResource(self._client)

    @cached_property
    def with_raw_response(self) -> BroadcastsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/zavudev/sdk-python#accessing-raw-response-data-eg-headers
        """
        return BroadcastsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> BroadcastsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/zavudev/sdk-python#with_streaming_response
        """
        return BroadcastsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        channel: BroadcastChannel,
        name: str,
        content: BroadcastContentParam | Omit = omit,
        email_html_body: str | Omit = omit,
        email_subject: str | Omit = omit,
        idempotency_key: str | Omit = omit,
        message_type: BroadcastMessageType | Omit = omit,
        metadata: Dict[str, str] | Omit = omit,
        scheduled_at: Union[str, datetime] | Omit = omit,
        sender_id: str | Omit = omit,
        text: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BroadcastCreateResponse:
        """Create a new broadcast campaign.

        Add contacts after creation, then send.

        Args:
          channel: Broadcast delivery channel. Use 'smart' for per-contact intelligent routing.

          name: Name of the broadcast campaign.

          content: Content for non-text broadcast message types.

          email_html_body: HTML body for email broadcasts.

          email_subject: Email subject line. Required for email broadcasts.

          idempotency_key: Idempotency key to prevent duplicate broadcasts.

          message_type: Type of message for broadcast.

          scheduled_at: Schedule the broadcast for future delivery.

          sender_id: Sender profile ID. Uses default sender if omitted.

          text: Text content or caption. Supports template variables: {{name}}, {{1}}, etc.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v1/broadcasts",
            body=maybe_transform(
                {
                    "channel": channel,
                    "name": name,
                    "content": content,
                    "email_html_body": email_html_body,
                    "email_subject": email_subject,
                    "idempotency_key": idempotency_key,
                    "message_type": message_type,
                    "metadata": metadata,
                    "scheduled_at": scheduled_at,
                    "sender_id": sender_id,
                    "text": text,
                },
                broadcast_create_params.BroadcastCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BroadcastCreateResponse,
        )

    def retrieve(
        self,
        broadcast_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BroadcastRetrieveResponse:
        """
        Get broadcast

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not broadcast_id:
            raise ValueError(f"Expected a non-empty value for `broadcast_id` but received {broadcast_id!r}")
        return self._get(
            f"/v1/broadcasts/{broadcast_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BroadcastRetrieveResponse,
        )

    def update(
        self,
        broadcast_id: str,
        *,
        content: BroadcastContentParam | Omit = omit,
        email_html_body: str | Omit = omit,
        email_subject: str | Omit = omit,
        metadata: Dict[str, str] | Omit = omit,
        name: str | Omit = omit,
        text: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BroadcastUpdateResponse:
        """
        Update a broadcast in draft status.

        Args:
          content: Content for non-text broadcast message types.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not broadcast_id:
            raise ValueError(f"Expected a non-empty value for `broadcast_id` but received {broadcast_id!r}")
        return self._patch(
            f"/v1/broadcasts/{broadcast_id}",
            body=maybe_transform(
                {
                    "content": content,
                    "email_html_body": email_html_body,
                    "email_subject": email_subject,
                    "metadata": metadata,
                    "name": name,
                    "text": text,
                },
                broadcast_update_params.BroadcastUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BroadcastUpdateResponse,
        )

    def list(
        self,
        *,
        cursor: str | Omit = omit,
        limit: int | Omit = omit,
        status: BroadcastStatus | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncCursor[Broadcast]:
        """
        List broadcasts for this project.

        Args:
          status: Current status of the broadcast.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/v1/broadcasts",
            page=SyncCursor[Broadcast],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "cursor": cursor,
                        "limit": limit,
                        "status": status,
                    },
                    broadcast_list_params.BroadcastListParams,
                ),
            ),
            model=Broadcast,
        )

    def delete(
        self,
        broadcast_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Delete a broadcast in draft status.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not broadcast_id:
            raise ValueError(f"Expected a non-empty value for `broadcast_id` but received {broadcast_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._delete(
            f"/v1/broadcasts/{broadcast_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    def cancel(
        self,
        broadcast_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BroadcastCancelResponse:
        """Cancel a broadcast.

        Pending contacts will be skipped, but already queued
        messages may still be delivered.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not broadcast_id:
            raise ValueError(f"Expected a non-empty value for `broadcast_id` but received {broadcast_id!r}")
        return self._post(
            f"/v1/broadcasts/{broadcast_id}/cancel",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BroadcastCancelResponse,
        )

    def progress(
        self,
        broadcast_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BroadcastProgress:
        """
        Get real-time progress of a broadcast including delivery counts and estimated
        completion time.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not broadcast_id:
            raise ValueError(f"Expected a non-empty value for `broadcast_id` but received {broadcast_id!r}")
        return self._get(
            f"/v1/broadcasts/{broadcast_id}/progress",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BroadcastProgress,
        )

    def reschedule(
        self,
        broadcast_id: str,
        *,
        scheduled_at: Union[str, datetime],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BroadcastRescheduleResponse:
        """Update the scheduled time for a broadcast.

        The broadcast must be in scheduled
        status.

        Args:
          scheduled_at: New scheduled time for the broadcast.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not broadcast_id:
            raise ValueError(f"Expected a non-empty value for `broadcast_id` but received {broadcast_id!r}")
        return self._patch(
            f"/v1/broadcasts/{broadcast_id}/schedule",
            body=maybe_transform({"scheduled_at": scheduled_at}, broadcast_reschedule_params.BroadcastRescheduleParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BroadcastRescheduleResponse,
        )

    def send(
        self,
        broadcast_id: str,
        *,
        scheduled_at: Union[str, datetime] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BroadcastSendResponse:
        """Start sending the broadcast immediately or schedule for later.

        Broadcasts go
        through automated AI content review before sending. If the review passes, the
        broadcast proceeds. If rejected, use PATCH to edit content, then call POST
        /retry-review. Reserves the estimated cost from your balance.

        Args:
          scheduled_at: Schedule for future delivery. Omit to send immediately.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not broadcast_id:
            raise ValueError(f"Expected a non-empty value for `broadcast_id` but received {broadcast_id!r}")
        return self._post(
            f"/v1/broadcasts/{broadcast_id}/send",
            body=maybe_transform({"scheduled_at": scheduled_at}, broadcast_send_params.BroadcastSendParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BroadcastSendResponse,
        )


class AsyncBroadcastsResource(AsyncAPIResource):
    @cached_property
    def contacts(self) -> AsyncContactsResource:
        return AsyncContactsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncBroadcastsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/zavudev/sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncBroadcastsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncBroadcastsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/zavudev/sdk-python#with_streaming_response
        """
        return AsyncBroadcastsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        channel: BroadcastChannel,
        name: str,
        content: BroadcastContentParam | Omit = omit,
        email_html_body: str | Omit = omit,
        email_subject: str | Omit = omit,
        idempotency_key: str | Omit = omit,
        message_type: BroadcastMessageType | Omit = omit,
        metadata: Dict[str, str] | Omit = omit,
        scheduled_at: Union[str, datetime] | Omit = omit,
        sender_id: str | Omit = omit,
        text: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BroadcastCreateResponse:
        """Create a new broadcast campaign.

        Add contacts after creation, then send.

        Args:
          channel: Broadcast delivery channel. Use 'smart' for per-contact intelligent routing.

          name: Name of the broadcast campaign.

          content: Content for non-text broadcast message types.

          email_html_body: HTML body for email broadcasts.

          email_subject: Email subject line. Required for email broadcasts.

          idempotency_key: Idempotency key to prevent duplicate broadcasts.

          message_type: Type of message for broadcast.

          scheduled_at: Schedule the broadcast for future delivery.

          sender_id: Sender profile ID. Uses default sender if omitted.

          text: Text content or caption. Supports template variables: {{name}}, {{1}}, etc.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v1/broadcasts",
            body=await async_maybe_transform(
                {
                    "channel": channel,
                    "name": name,
                    "content": content,
                    "email_html_body": email_html_body,
                    "email_subject": email_subject,
                    "idempotency_key": idempotency_key,
                    "message_type": message_type,
                    "metadata": metadata,
                    "scheduled_at": scheduled_at,
                    "sender_id": sender_id,
                    "text": text,
                },
                broadcast_create_params.BroadcastCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BroadcastCreateResponse,
        )

    async def retrieve(
        self,
        broadcast_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BroadcastRetrieveResponse:
        """
        Get broadcast

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not broadcast_id:
            raise ValueError(f"Expected a non-empty value for `broadcast_id` but received {broadcast_id!r}")
        return await self._get(
            f"/v1/broadcasts/{broadcast_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BroadcastRetrieveResponse,
        )

    async def update(
        self,
        broadcast_id: str,
        *,
        content: BroadcastContentParam | Omit = omit,
        email_html_body: str | Omit = omit,
        email_subject: str | Omit = omit,
        metadata: Dict[str, str] | Omit = omit,
        name: str | Omit = omit,
        text: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BroadcastUpdateResponse:
        """
        Update a broadcast in draft status.

        Args:
          content: Content for non-text broadcast message types.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not broadcast_id:
            raise ValueError(f"Expected a non-empty value for `broadcast_id` but received {broadcast_id!r}")
        return await self._patch(
            f"/v1/broadcasts/{broadcast_id}",
            body=await async_maybe_transform(
                {
                    "content": content,
                    "email_html_body": email_html_body,
                    "email_subject": email_subject,
                    "metadata": metadata,
                    "name": name,
                    "text": text,
                },
                broadcast_update_params.BroadcastUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BroadcastUpdateResponse,
        )

    def list(
        self,
        *,
        cursor: str | Omit = omit,
        limit: int | Omit = omit,
        status: BroadcastStatus | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[Broadcast, AsyncCursor[Broadcast]]:
        """
        List broadcasts for this project.

        Args:
          status: Current status of the broadcast.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/v1/broadcasts",
            page=AsyncCursor[Broadcast],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "cursor": cursor,
                        "limit": limit,
                        "status": status,
                    },
                    broadcast_list_params.BroadcastListParams,
                ),
            ),
            model=Broadcast,
        )

    async def delete(
        self,
        broadcast_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Delete a broadcast in draft status.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not broadcast_id:
            raise ValueError(f"Expected a non-empty value for `broadcast_id` but received {broadcast_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._delete(
            f"/v1/broadcasts/{broadcast_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    async def cancel(
        self,
        broadcast_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BroadcastCancelResponse:
        """Cancel a broadcast.

        Pending contacts will be skipped, but already queued
        messages may still be delivered.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not broadcast_id:
            raise ValueError(f"Expected a non-empty value for `broadcast_id` but received {broadcast_id!r}")
        return await self._post(
            f"/v1/broadcasts/{broadcast_id}/cancel",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BroadcastCancelResponse,
        )

    async def progress(
        self,
        broadcast_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BroadcastProgress:
        """
        Get real-time progress of a broadcast including delivery counts and estimated
        completion time.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not broadcast_id:
            raise ValueError(f"Expected a non-empty value for `broadcast_id` but received {broadcast_id!r}")
        return await self._get(
            f"/v1/broadcasts/{broadcast_id}/progress",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BroadcastProgress,
        )

    async def reschedule(
        self,
        broadcast_id: str,
        *,
        scheduled_at: Union[str, datetime],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BroadcastRescheduleResponse:
        """Update the scheduled time for a broadcast.

        The broadcast must be in scheduled
        status.

        Args:
          scheduled_at: New scheduled time for the broadcast.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not broadcast_id:
            raise ValueError(f"Expected a non-empty value for `broadcast_id` but received {broadcast_id!r}")
        return await self._patch(
            f"/v1/broadcasts/{broadcast_id}/schedule",
            body=await async_maybe_transform(
                {"scheduled_at": scheduled_at}, broadcast_reschedule_params.BroadcastRescheduleParams
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BroadcastRescheduleResponse,
        )

    async def send(
        self,
        broadcast_id: str,
        *,
        scheduled_at: Union[str, datetime] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BroadcastSendResponse:
        """Start sending the broadcast immediately or schedule for later.

        Broadcasts go
        through automated AI content review before sending. If the review passes, the
        broadcast proceeds. If rejected, use PATCH to edit content, then call POST
        /retry-review. Reserves the estimated cost from your balance.

        Args:
          scheduled_at: Schedule for future delivery. Omit to send immediately.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not broadcast_id:
            raise ValueError(f"Expected a non-empty value for `broadcast_id` but received {broadcast_id!r}")
        return await self._post(
            f"/v1/broadcasts/{broadcast_id}/send",
            body=await async_maybe_transform({"scheduled_at": scheduled_at}, broadcast_send_params.BroadcastSendParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BroadcastSendResponse,
        )


class BroadcastsResourceWithRawResponse:
    def __init__(self, broadcasts: BroadcastsResource) -> None:
        self._broadcasts = broadcasts

        self.create = to_raw_response_wrapper(
            broadcasts.create,
        )
        self.retrieve = to_raw_response_wrapper(
            broadcasts.retrieve,
        )
        self.update = to_raw_response_wrapper(
            broadcasts.update,
        )
        self.list = to_raw_response_wrapper(
            broadcasts.list,
        )
        self.delete = to_raw_response_wrapper(
            broadcasts.delete,
        )
        self.cancel = to_raw_response_wrapper(
            broadcasts.cancel,
        )
        self.progress = to_raw_response_wrapper(
            broadcasts.progress,
        )
        self.reschedule = to_raw_response_wrapper(
            broadcasts.reschedule,
        )
        self.send = to_raw_response_wrapper(
            broadcasts.send,
        )

    @cached_property
    def contacts(self) -> ContactsResourceWithRawResponse:
        return ContactsResourceWithRawResponse(self._broadcasts.contacts)


class AsyncBroadcastsResourceWithRawResponse:
    def __init__(self, broadcasts: AsyncBroadcastsResource) -> None:
        self._broadcasts = broadcasts

        self.create = async_to_raw_response_wrapper(
            broadcasts.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            broadcasts.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            broadcasts.update,
        )
        self.list = async_to_raw_response_wrapper(
            broadcasts.list,
        )
        self.delete = async_to_raw_response_wrapper(
            broadcasts.delete,
        )
        self.cancel = async_to_raw_response_wrapper(
            broadcasts.cancel,
        )
        self.progress = async_to_raw_response_wrapper(
            broadcasts.progress,
        )
        self.reschedule = async_to_raw_response_wrapper(
            broadcasts.reschedule,
        )
        self.send = async_to_raw_response_wrapper(
            broadcasts.send,
        )

    @cached_property
    def contacts(self) -> AsyncContactsResourceWithRawResponse:
        return AsyncContactsResourceWithRawResponse(self._broadcasts.contacts)


class BroadcastsResourceWithStreamingResponse:
    def __init__(self, broadcasts: BroadcastsResource) -> None:
        self._broadcasts = broadcasts

        self.create = to_streamed_response_wrapper(
            broadcasts.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            broadcasts.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            broadcasts.update,
        )
        self.list = to_streamed_response_wrapper(
            broadcasts.list,
        )
        self.delete = to_streamed_response_wrapper(
            broadcasts.delete,
        )
        self.cancel = to_streamed_response_wrapper(
            broadcasts.cancel,
        )
        self.progress = to_streamed_response_wrapper(
            broadcasts.progress,
        )
        self.reschedule = to_streamed_response_wrapper(
            broadcasts.reschedule,
        )
        self.send = to_streamed_response_wrapper(
            broadcasts.send,
        )

    @cached_property
    def contacts(self) -> ContactsResourceWithStreamingResponse:
        return ContactsResourceWithStreamingResponse(self._broadcasts.contacts)


class AsyncBroadcastsResourceWithStreamingResponse:
    def __init__(self, broadcasts: AsyncBroadcastsResource) -> None:
        self._broadcasts = broadcasts

        self.create = async_to_streamed_response_wrapper(
            broadcasts.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            broadcasts.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            broadcasts.update,
        )
        self.list = async_to_streamed_response_wrapper(
            broadcasts.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            broadcasts.delete,
        )
        self.cancel = async_to_streamed_response_wrapper(
            broadcasts.cancel,
        )
        self.progress = async_to_streamed_response_wrapper(
            broadcasts.progress,
        )
        self.reschedule = async_to_streamed_response_wrapper(
            broadcasts.reschedule,
        )
        self.send = async_to_streamed_response_wrapper(
            broadcasts.send,
        )

    @cached_property
    def contacts(self) -> AsyncContactsResourceWithStreamingResponse:
        return AsyncContactsResourceWithStreamingResponse(self._broadcasts.contacts)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["RegulatoryDocumentCreateParams"]


class RegulatoryDocumentCreateParams(TypedDict, total=False):
    document_type: Required[
        Annotated[
            Literal[
                "passport",
                "national_id",
                "drivers_license",
                "utility_bill",
                "tax_id",
                "business_registration",
                "proof_of_address",
                "other",
            ],
            PropertyInfo(alias="documentType"),
        ]
    ]

    file_size: Required[Annotated[int, PropertyInfo(alias="fileSize")]]

    mime_type: Required[Annotated[str, PropertyInfo(alias="mimeType")]]

    name: Required[str]

    storage_id: Required[Annotated[str, PropertyInfo(alias="storageId")]]
    """Storage ID from the upload-url endpoint."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Optional
from datetime import datetime
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["Contact", "Channel", "ChannelMetrics"]


class ChannelMetrics(BaseModel):
    """Delivery metrics for this channel."""

    avg_delivery_time_ms: Optional[float] = FieldInfo(alias="avgDeliveryTimeMs", default=None)

    failure_count: Optional[int] = FieldInfo(alias="failureCount", default=None)

    last_success_at: Optional[datetime] = FieldInfo(alias="lastSuccessAt", default=None)

    success_count: Optional[int] = FieldInfo(alias="successCount", default=None)

    total_attempts: Optional[int] = FieldInfo(alias="totalAttempts", default=None)


class Channel(BaseModel):
    """A communication channel for a contact."""

    id: str

    channel: Literal["sms", "whatsapp", "email", "telegram", "voice"]
    """Channel type."""

    created_at: datetime = FieldInfo(alias="createdAt")

    identifier: str
    """Channel identifier (phone number or email address)."""

    is_primary: bool = FieldInfo(alias="isPrimary")
    """Whether this is the primary channel for its type."""

    verified: bool
    """Whether this channel has been verified."""

    country_code: Optional[str] = FieldInfo(alias="countryCode", default=None)
    """ISO country code for phone numbers."""

    label: Optional[str] = None
    """Optional label for the channel."""

    last_inbound_at: Optional[datetime] = FieldInfo(alias="lastInboundAt", default=None)
    """Last time a message was received on this channel."""

    metadata: Optional[Dict[str, str]] = None

    metrics: Optional[ChannelMetrics] = None
    """Delivery metrics for this channel."""

    updated_at: Optional[datetime] = FieldInfo(alias="updatedAt", default=None)


class Contact(BaseModel):
    id: str

    available_channels: List[str] = FieldInfo(alias="availableChannels")
    """List of available messaging channels for this contact."""

    created_at: datetime = FieldInfo(alias="createdAt")

    metadata: Dict[str, str]

    verified: bool
    """Whether this contact has been verified."""

    channels: Optional[List[Channel]] = None
    """All communication channels for this contact."""

    country_code: Optional[str] = FieldInfo(alias="countryCode", default=None)

    default_channel: Optional[Literal["sms", "whatsapp", "telegram", "email", "instagram", "voice"]] = FieldInfo(
        alias="defaultChannel", default=None
    )
    """Preferred channel for this contact."""

    display_name: Optional[str] = FieldInfo(alias="displayName", default=None)
    """Display name for the contact."""

    phone_number: Optional[str] = FieldInfo(alias="phoneNumber", default=None)
    """DEPRECATED: Use primaryPhone instead. Primary phone number in E.164 format."""

    primary_email: Optional[str] = FieldInfo(alias="primaryEmail", default=None)
    """Primary email address."""

    primary_phone: Optional[str] = FieldInfo(alias="primaryPhone", default=None)
    """Primary phone number in E.164 format."""

    profile_name: Optional[str] = FieldInfo(alias="profileName", default=None)
    """Contact's WhatsApp profile name. Only available for WhatsApp contacts."""

    suggested_merge_with: Optional[str] = FieldInfo(alias="suggestedMergeWith", default=None)
    """ID of a contact suggested for merging."""

    updated_at: Optional[datetime] = FieldInfo(alias="updatedAt", default=None)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from datetime import datetime
from typing_extensions import Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["BroadcastSendParams"]


class BroadcastSendParams(TypedDict, total=False):
    scheduled_at: Annotated[Union[str, datetime], PropertyInfo(alias="scheduledAt", format="iso8601")]
    """Schedule for future delivery. Omit to send immediately."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Iterable
from typing_extensions import Required, Annotated, TypedDict

from .._types import SequenceNotStr
from .._utils import PropertyInfo

__all__ = ["MessageContentParam", "Button", "Contact", "Section", "SectionRow"]


class Button(TypedDict, total=False):
    id: Required[str]

    title: Required[str]


class Contact(TypedDict, total=False):
    name: str

    phones: SequenceNotStr[str]


class SectionRow(TypedDict, total=False):
    id: Required[str]

    title: Required[str]

    description: str


class Section(TypedDict, total=False):
    rows: Required[Iterable[SectionRow]]

    title: Required[str]


class MessageContentParam(TypedDict, total=False):
    """Content for non-text message types (WhatsApp and Telegram)."""

    buttons: Iterable[Button]
    """Interactive buttons (max 3)."""

    contacts: Iterable[Contact]
    """Contact cards for contact messages."""

    emoji: str
    """Emoji for reaction messages."""

    filename: str
    """Filename for documents."""

    latitude: float
    """Latitude for location messages."""

    list_button: Annotated[str, PropertyInfo(alias="listButton")]
    """Button text for list messages."""

    location_address: Annotated[str, PropertyInfo(alias="locationAddress")]
    """Address of the location."""

    location_name: Annotated[str, PropertyInfo(alias="locationName")]
    """Name of the location."""

    longitude: float
    """Longitude for location messages."""

    media_id: Annotated[str, PropertyInfo(alias="mediaId")]
    """WhatsApp media ID if already uploaded."""

    media_url: Annotated[str, PropertyInfo(alias="mediaUrl")]
    """URL of the media file (for image, video, audio, document, sticker)."""

    mime_type: Annotated[str, PropertyInfo(alias="mimeType")]
    """MIME type of the media."""

    react_to_message_id: Annotated[str, PropertyInfo(alias="reactToMessageId")]
    """Message ID to react to."""

    sections: Iterable[Section]
    """Sections for list messages."""

    template_id: Annotated[str, PropertyInfo(alias="templateId")]
    """Template ID for template messages."""

    template_variables: Annotated[Dict[str, str], PropertyInfo(alias="templateVariables")]
    """Variables for template rendering. Keys are variable positions (1, 2, 3...)."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable

import httpx

from ...types import BroadcastContactStatus
from ..._types import Body, Omit, Query, Headers, NoneType, NotGiven, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...pagination import SyncCursor, AsyncCursor
from ..._base_client import AsyncPaginator, make_request_options
from ...types.broadcasts import contact_add_params, contact_list_params
from ...types.broadcast_contact import BroadcastContact
from ...types.broadcast_contact_status import BroadcastContactStatus
from ...types.broadcasts.contact_add_response import ContactAddResponse

__all__ = ["ContactsResource", "AsyncContactsResource"]


class ContactsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ContactsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/zavudev/sdk-python#accessing-raw-response-data-eg-headers
        """
        return ContactsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ContactsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/zavudev/sdk-python#with_streaming_response
        """
        return ContactsResourceWithStreamingResponse(self)

    def list(
        self,
        broadcast_id: str,
        *,
        cursor: str | Omit = omit,
        limit: int | Omit = omit,
        status: BroadcastContactStatus | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncCursor[BroadcastContact]:
        """
        List contacts in a broadcast with optional status filter.

        Args:
          status: Status of a contact within a broadcast.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not broadcast_id:
            raise ValueError(f"Expected a non-empty value for `broadcast_id` but received {broadcast_id!r}")
        return self._get_api_list(
            f"/v1/broadcasts/{broadcast_id}/contacts",
            page=SyncCursor[BroadcastContact],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "cursor": cursor,
                        "limit": limit,
                        "status": status,
                    },
                    contact_list_params.ContactListParams,
                ),
            ),
            model=BroadcastContact,
        )

    def add(
        self,
        broadcast_id: str,
        *,
        contacts: Iterable[contact_add_params.Contact],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ContactAddResponse:
        """Add contacts to a broadcast in batch.

        Maximum 1000 contacts per request.

        Args:
          contacts: List of contacts to add (max 1000 per request).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not broadcast_id:
            raise ValueError(f"Expected a non-empty value for `broadcast_id` but received {broadcast_id!r}")
        return self._post(
            f"/v1/broadcasts/{broadcast_id}/contacts",
            body=maybe_transform({"contacts": contacts}, contact_add_params.ContactAddParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ContactAddResponse,
        )

    def remove(
        self,
        contact_id: str,
        *,
        broadcast_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Remove a contact from a broadcast in draft status.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not broadcast_id:
            raise ValueError(f"Expected a non-empty value for `broadcast_id` but received {broadcast_id!r}")
        if not contact_id:
            raise ValueError(f"Expected a non-empty value for `contact_id` but received {contact_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._delete(
            f"/v1/broadcasts/{broadcast_id}/contacts/{contact_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class AsyncContactsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncContactsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/zavudev/sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncContactsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncContactsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/zavudev/sdk-python#with_streaming_response
        """
        return AsyncContactsResourceWithStreamingResponse(self)

    def list(
        self,
        broadcast_id: str,
        *,
        cursor: str | Omit = omit,
        limit: int | Omit = omit,
        status: BroadcastContactStatus | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[BroadcastContact, AsyncCursor[BroadcastContact]]:
        """
        List contacts in a broadcast with optional status filter.

        Args:
          status: Status of a contact within a broadcast.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not broadcast_id:
            raise ValueError(f"Expected a non-empty value for `broadcast_id` but received {broadcast_id!r}")
        return self._get_api_list(
            f"/v1/broadcasts/{broadcast_id}/contacts",
            page=AsyncCursor[BroadcastContact],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "cursor": cursor,
                        "limit": limit,
                        "status": status,
                    },
                    contact_list_params.ContactListParams,
                ),
            ),
            model=BroadcastContact,
        )

    async def add(
        self,
        broadcast_id: str,
        *,
        contacts: Iterable[contact_add_params.Contact],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ContactAddResponse:
        """Add contacts to a broadcast in batch.

        Maximum 1000 contacts per request.

        Args:
          contacts: List of contacts to add (max 1000 per request).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not broadcast_id:
            raise ValueError(f"Expected a non-empty value for `broadcast_id` but received {broadcast_id!r}")
        return await self._post(
            f"/v1/broadcasts/{broadcast_id}/contacts",
            body=await async_maybe_transform({"contacts": contacts}, contact_add_params.ContactAddParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ContactAddResponse,
        )

    async def remove(
        self,
        contact_id: str,
        *,
        broadcast_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Remove a contact from a broadcast in draft status.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not broadcast_id:
            raise ValueError(f"Expected a non-empty value for `broadcast_id` but received {broadcast_id!r}")
        if not contact_id:
            raise ValueError(f"Expected a non-empty value for `contact_id` but received {contact_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._delete(
            f"/v1/broadcasts/{broadcast_id}/contacts/{contact_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class ContactsResourceWithRawResponse:
    def __init__(self, contacts: ContactsResource) -> None:
        self._contacts = contacts

        self.list = to_raw_response_wrapper(
            contacts.list,
        )
        self.add = to_raw_response_wrapper(
            contacts.add,
        )
        self.remove = to_raw_response_wrapper(
            contacts.remove,
        )


class AsyncContactsResourceWithRawResponse:
    def __init__(self, contacts: AsyncContactsResource) -> None:
        self._contacts = contacts

        self.list = async_to_raw_response_wrapper(
            contacts.list,
        )
        self.add = async_to_raw_response_wrapper(
            contacts.add,
        )
        self.remove = async_to_raw_response_wrapper(
            contacts.remove,
        )


class ContactsResourceWithStreamingResponse:
    def __init__(self, contacts: ContactsResource) -> None:
        self._contacts = contacts

        self.list = to_streamed_response_wrapper(
            contacts.list,
        )
        self.add = to_streamed_response_wrapper(
            contacts.add,
        )
        self.remove = to_streamed_response_wrapper(
            contacts.remove,
        )


class AsyncContactsResourceWithStreamingResponse:
    def __init__(self, contacts: AsyncContactsResource) -> None:
        self._contacts = contacts

        self.list = async_to_streamed_response_wrapper(
            contacts.list,
        )
        self.add = async_to_streamed_response_wrapper(
            contacts.add,
        )
        self.remove = async_to_streamed_response_wrapper(
            contacts.remove,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal, TypeAlias

__all__ = ["LineType"]

LineType: TypeAlias = Literal["mobile", "landline", "voip", "toll_free", "unknown"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, Annotated, TypedDict

from ...._utils import PropertyInfo

__all__ = ["FlowDuplicateParams"]


class FlowDuplicateParams(TypedDict, total=False):
    sender_id: Required[Annotated[str, PropertyInfo(alias="senderId")]]

    new_name: Required[Annotated[str, PropertyInfo(alias="newName")]]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

from .channel import Channel
from .message_status import MessageStatus

__all__ = ["MessageListParams"]


class MessageListParams(TypedDict, total=False):
    channel: Channel
    """Delivery channel. Use 'auto' for intelligent routing."""

    cursor: str

    limit: int

    status: MessageStatus

    to: str



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["RegulatoryDocumentUploadURLResponse"]


class RegulatoryDocumentUploadURLResponse(BaseModel):
    upload_url: str = FieldInfo(alias="uploadUrl")
    """Pre-signed URL for uploading the file."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .._models import BaseModel
from .regulatory_document import RegulatoryDocument

__all__ = ["RegulatoryDocumentRetrieveResponse"]


class RegulatoryDocumentRetrieveResponse(BaseModel):
    document: RegulatoryDocument
    """A regulatory document for phone number requirements."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Optional
from datetime import datetime
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from ...._models import BaseModel

__all__ = ["AgentTool", "Parameters", "ParametersProperties"]


class ParametersProperties(BaseModel):
    description: Optional[str] = None

    type: Optional[str] = None


class Parameters(BaseModel):
    properties: Dict[str, ParametersProperties]

    required: List[str]

    type: Literal["object"]


class AgentTool(BaseModel):
    id: str

    agent_id: str = FieldInfo(alias="agentId")

    created_at: datetime = FieldInfo(alias="createdAt")

    description: str
    """Description for the LLM to understand when to use this tool."""

    enabled: bool

    name: str

    parameters: Parameters

    updated_at: datetime = FieldInfo(alias="updatedAt")

    webhook_url: str = FieldInfo(alias="webhookUrl")
    """HTTPS URL to call when the tool is executed."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from .._models import BaseModel
from .requirement import Requirement

__all__ = ["PhoneNumberRequirementsResponse"]


class PhoneNumberRequirementsResponse(BaseModel):
    items: List[Requirement]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

from .phone_number_status import PhoneNumberStatus

__all__ = ["PhoneNumberListParams"]


class PhoneNumberListParams(TypedDict, total=False):
    cursor: str
    """Pagination cursor."""

    limit: int

    status: PhoneNumberStatus
    """Filter by phone number status."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict
from typing_extensions import Annotated, TypedDict

from .._utils import PropertyInfo
from .broadcast_content_param import BroadcastContentParam

__all__ = ["BroadcastUpdateParams"]


class BroadcastUpdateParams(TypedDict, total=False):
    content: BroadcastContentParam
    """Content for non-text broadcast message types."""

    email_html_body: Annotated[str, PropertyInfo(alias="emailHtmlBody")]

    email_subject: Annotated[str, PropertyInfo(alias="emailSubject")]

    metadata: Dict[str, str]

    name: str

    text: str



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

from .broadcast_status import BroadcastStatus

__all__ = ["BroadcastListParams"]


class BroadcastListParams(TypedDict, total=False):
    cursor: str

    limit: int

    status: BroadcastStatus
    """Current status of the broadcast."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from pydantic import Field as FieldInfo

from .._models import BaseModel
from .line_type import LineType

__all__ = ["IntrospectValidatePhoneResponse", "Carrier"]


class Carrier(BaseModel):
    """Carrier information for the phone number."""

    name: Optional[str] = None
    """Carrier name."""

    type: Optional[LineType] = None
    """Type of phone line."""


class IntrospectValidatePhoneResponse(BaseModel):
    country_code: str = FieldInfo(alias="countryCode")

    phone_number: str = FieldInfo(alias="phoneNumber")

    valid_number: bool = FieldInfo(alias="validNumber")

    available_channels: Optional[List[str]] = FieldInfo(alias="availableChannels", default=None)
    """List of available messaging channels for this phone number."""

    carrier: Optional[Carrier] = None
    """Carrier information for the phone number."""

    line_type: Optional[LineType] = FieldInfo(alias="lineType", default=None)
    """Type of phone line."""

    national_format: Optional[str] = FieldInfo(alias="nationalFormat", default=None)
    """Phone number in national format."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from .._models import BaseModel
from .webhook_event import WebhookEvent

__all__ = ["SenderWebhook"]


class SenderWebhook(BaseModel):
    """Webhook configuration for the sender."""

    active: bool
    """Whether the webhook is active."""

    events: List[WebhookEvent]
    """List of events the webhook is subscribed to."""

    url: str
    """HTTPS URL that will receive webhook events."""

    secret: Optional[str] = None
    """Webhook secret for signature verification.

    Only returned on create or regenerate.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .message import Message
from .._models import BaseModel

__all__ = ["MessageResponse"]


class MessageResponse(BaseModel):
    message: Message



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from zavudev import Zavudev, AsyncZavudev
from tests.utils import assert_matches_type
from zavudev.types import (
    Broadcast,
    BroadcastProgress,
    BroadcastSendResponse,
    BroadcastCancelResponse,
    BroadcastCreateResponse,
    BroadcastUpdateResponse,
    BroadcastRetrieveResponse,
    BroadcastRescheduleResponse,
)
from zavudev._utils import parse_datetime
from zavudev.pagination import SyncCursor, AsyncCursor

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestBroadcasts:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create(self, client: Zavudev) -> None:
        broadcast = client.broadcasts.create(
            channel="sms",
            name="Black Friday Sale",
        )
        assert_matches_type(BroadcastCreateResponse, broadcast, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create_with_all_params(self, client: Zavudev) -> None:
        broadcast = client.broadcasts.create(
            channel="sms",
            name="Black Friday Sale",
            content={
                "filename": "filename",
                "media_id": "mediaId",
                "media_url": "mediaUrl",
                "mime_type": "mimeType",
                "template_id": "templateId",
                "template_variables": {"foo": "string"},
            },
            email_html_body="emailHtmlBody",
            email_subject="emailSubject",
            idempotency_key="idempotencyKey",
            message_type="text",
            metadata={"foo": "string"},
            scheduled_at=parse_datetime("2019-12-27T18:11:19.117Z"),
            sender_id="senderId",
            text="Hi {{name}}, check out our Black Friday deals! Use code FRIDAY20 for 20% off.",
        )
        assert_matches_type(BroadcastCreateResponse, broadcast, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_create(self, client: Zavudev) -> None:
        response = client.broadcasts.with_raw_response.create(
            channel="sms",
            name="Black Friday Sale",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        broadcast = response.parse()
        assert_matches_type(BroadcastCreateResponse, broadcast, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_create(self, client: Zavudev) -> None:
        with client.broadcasts.with_streaming_response.create(
            channel="sms",
            name="Black Friday Sale",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            broadcast = response.parse()
            assert_matches_type(BroadcastCreateResponse, broadcast, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Zavudev) -> None:
        broadcast = client.broadcasts.retrieve(
            "broadcastId",
        )
        assert_matches_type(BroadcastRetrieveResponse, broadcast, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Zavudev) -> None:
        response = client.broadcasts.with_raw_response.retrieve(
            "broadcastId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        broadcast = response.parse()
        assert_matches_type(BroadcastRetrieveResponse, broadcast, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Zavudev) -> None:
        with client.broadcasts.with_streaming_response.retrieve(
            "broadcastId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            broadcast = response.parse()
            assert_matches_type(BroadcastRetrieveResponse, broadcast, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Zavudev) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `broadcast_id` but received ''"):
            client.broadcasts.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_update(self, client: Zavudev) -> None:
        broadcast = client.broadcasts.update(
            broadcast_id="broadcastId",
        )
        assert_matches_type(BroadcastUpdateResponse, broadcast, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_update_with_all_params(self, client: Zavudev) -> None:
        broadcast = client.broadcasts.update(
            broadcast_id="broadcastId",
            content={
                "filename": "filename",
                "media_id": "mediaId",
                "media_url": "mediaUrl",
                "mime_type": "mimeType",
                "template_id": "templateId",
                "template_variables": {"foo": "string"},
            },
            email_html_body="emailHtmlBody",
            email_subject="emailSubject",
            metadata={"foo": "string"},
            name="name",
            text="text",
        )
        assert_matches_type(BroadcastUpdateResponse, broadcast, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_update(self, client: Zavudev) -> None:
        response = client.broadcasts.with_raw_response.update(
            broadcast_id="broadcastId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        broadcast = response.parse()
        assert_matches_type(BroadcastUpdateResponse, broadcast, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_update(self, client: Zavudev) -> None:
        with client.broadcasts.with_streaming_response.update(
            broadcast_id="broadcastId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            broadcast = response.parse()
            assert_matches_type(BroadcastUpdateResponse, broadcast, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_update(self, client: Zavudev) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `broadcast_id` but received ''"):
            client.broadcasts.with_raw_response.update(
                broadcast_id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list(self, client: Zavudev) -> None:
        broadcast = client.broadcasts.list()
        assert_matches_type(SyncCursor[Broadcast], broadcast, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: Zavudev) -> None:
        broadcast = client.broadcasts.list(
            cursor="cursor",
            limit=100,
            status="draft",
        )
        assert_matches_type(SyncCursor[Broadcast], broadcast, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Zavudev) -> None:
        response = client.broadcasts.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        broadcast = response.parse()
        assert_matches_type(SyncCursor[Broadcast], broadcast, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Zavudev) -> None:
        with client.broadcasts.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            broadcast = response.parse()
            assert_matches_type(SyncCursor[Broadcast], broadcast, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_delete(self, client: Zavudev) -> None:
        broadcast = client.broadcasts.delete(
            "broadcastId",
        )
        assert broadcast is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_delete(self, client: Zavudev) -> None:
        response = client.broadcasts.with_raw_response.delete(
            "broadcastId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        broadcast = response.parse()
        assert broadcast is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_delete(self, client: Zavudev) -> None:
        with client.broadcasts.with_streaming_response.delete(
            "broadcastId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            broadcast = response.parse()
            assert broadcast is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_delete(self, client: Zavudev) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `broadcast_id` but received ''"):
            client.broadcasts.with_raw_response.delete(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_cancel(self, client: Zavudev) -> None:
        broadcast = client.broadcasts.cancel(
            "broadcastId",
        )
        assert_matches_type(BroadcastCancelResponse, broadcast, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_cancel(self, client: Zavudev) -> None:
        response = client.broadcasts.with_raw_response.cancel(
            "broadcastId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        broadcast = response.parse()
        assert_matches_type(BroadcastCancelResponse, broadcast, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_cancel(self, client: Zavudev) -> None:
        with client.broadcasts.with_streaming_response.cancel(
            "broadcastId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            broadcast = response.parse()
            assert_matches_type(BroadcastCancelResponse, broadcast, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_cancel(self, client: Zavudev) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `broadcast_id` but received ''"):
            client.broadcasts.with_raw_response.cancel(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_progress(self, client: Zavudev) -> None:
        broadcast = client.broadcasts.progress(
            "broadcastId",
        )
        assert_matches_type(BroadcastProgress, broadcast, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_progress(self, client: Zavudev) -> None:
        response = client.broadcasts.with_raw_response.progress(
            "broadcastId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        broadcast = response.parse()
        assert_matches_type(BroadcastProgress, broadcast, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_progress(self, client: Zavudev) -> None:
        with client.broadcasts.with_streaming_response.progress(
            "broadcastId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            broadcast = response.parse()
            assert_matches_type(BroadcastProgress, broadcast, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_progress(self, client: Zavudev) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `broadcast_id` but received ''"):
            client.broadcasts.with_raw_response.progress(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_reschedule(self, client: Zavudev) -> None:
        broadcast = client.broadcasts.reschedule(
            broadcast_id="broadcastId",
            scheduled_at=parse_datetime("2024-01-15T14:00:00Z"),
        )
        assert_matches_type(BroadcastRescheduleResponse, broadcast, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_reschedule(self, client: Zavudev) -> None:
        response = client.broadcasts.with_raw_response.reschedule(
            broadcast_id="broadcastId",
            scheduled_at=parse_datetime("2024-01-15T14:00:00Z"),
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        broadcast = response.parse()
        assert_matches_type(BroadcastRescheduleResponse, broadcast, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_reschedule(self, client: Zavudev) -> None:
        with client.broadcasts.with_streaming_response.reschedule(
            broadcast_id="broadcastId",
            scheduled_at=parse_datetime("2024-01-15T14:00:00Z"),
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            broadcast = response.parse()
            assert_matches_type(BroadcastRescheduleResponse, broadcast, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_reschedule(self, client: Zavudev) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `broadcast_id` but received ''"):
            client.broadcasts.with_raw_response.reschedule(
                broadcast_id="",
                scheduled_at=parse_datetime("2024-01-15T14:00:00Z"),
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_send(self, client: Zavudev) -> None:
        broadcast = client.broadcasts.send(
            broadcast_id="broadcastId",
        )
        assert_matches_type(BroadcastSendResponse, broadcast, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_send_with_all_params(self, client: Zavudev) -> None:
        broadcast = client.broadcasts.send(
            broadcast_id="broadcastId",
            scheduled_at=parse_datetime("2019-12-27T18:11:19.117Z"),
        )
        assert_matches_type(BroadcastSendResponse, broadcast, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_send(self, client: Zavudev) -> None:
        response = client.broadcasts.with_raw_response.send(
            broadcast_id="broadcastId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        broadcast = response.parse()
        assert_matches_type(BroadcastSendResponse, broadcast, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_send(self, client: Zavudev) -> None:
        with client.broadcasts.with_streaming_response.send(
            broadcast_id="broadcastId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            broadcast = response.parse()
            assert_matches_type(BroadcastSendResponse, broadcast, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_send(self, client: Zavudev) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `broadcast_id` but received ''"):
            client.broadcasts.with_raw_response.send(
                broadcast_id="",
            )


class TestAsyncBroadcasts:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create(self, async_client: AsyncZavudev) -> None:
        broadcast = await async_client.broadcasts.create(
            channel="sms",
            name="Black Friday Sale",
        )
        assert_matches_type(BroadcastCreateResponse, broadcast, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncZavudev) -> None:
        broadcast = await async_client.broadcasts.create(
            channel="sms",
            name="Black Friday Sale",
            content={
                "filename": "filename",
                "media_id": "mediaId",
                "media_url": "mediaUrl",
                "mime_type": "mimeType",
                "template_id": "templateId",
                "template_variables": {"foo": "string"},
            },
            email_html_body="emailHtmlBody",
            email_subject="emailSubject",
            idempotency_key="idempotencyKey",
            message_type="text",
            metadata={"foo": "string"},
            scheduled_at=parse_datetime("2019-12-27T18:11:19.117Z"),
            sender_id="senderId",
            text="Hi {{name}}, check out our Black Friday deals! Use code FRIDAY20 for 20% off.",
        )
        assert_matches_type(BroadcastCreateResponse, broadcast, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncZavudev) -> None:
        response = await async_client.broadcasts.with_raw_response.create(
            channel="sms",
            name="Black Friday Sale",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        broadcast = await response.parse()
        assert_matches_type(BroadcastCreateResponse, broadcast, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncZavudev) -> None:
        async with async_client.broadcasts.with_streaming_response.create(
            channel="sms",
            name="Black Friday Sale",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            broadcast = await response.parse()
            assert_matches_type(BroadcastCreateResponse, broadcast, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncZavudev) -> None:
        broadcast = await async_client.broadcasts.retrieve(
            "broadcastId",
        )
        assert_matches_type(BroadcastRetrieveResponse, broadcast, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncZavudev) -> None:
        response = await async_client.broadcasts.with_raw_response.retrieve(
            "broadcastId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        broadcast = await response.parse()
        assert_matches_type(BroadcastRetrieveResponse, broadcast, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncZavudev) -> None:
        async with async_client.broadcasts.with_streaming_response.retrieve(
            "broadcastId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            broadcast = await response.parse()
            assert_matches_type(BroadcastRetrieveResponse, broadcast, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncZavudev) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `broadcast_id` but received ''"):
            await async_client.broadcasts.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_update(self, async_client: AsyncZavudev) -> None:
        broadcast = await async_client.broadcasts.update(
            broadcast_id="broadcastId",
        )
        assert_matches_type(BroadcastUpdateResponse, broadcast, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncZavudev) -> None:
        broadcast = await async_client.broadcasts.update(
            broadcast_id="broadcastId",
            content={
                "filename": "filename",
                "media_id": "mediaId",
                "media_url": "mediaUrl",
                "mime_type": "mimeType",
                "template_id": "templateId",
                "template_variables": {"foo": "string"},
            },
            email_html_body="emailHtmlBody",
            email_subject="emailSubject",
            metadata={"foo": "string"},
            name="name",
            text="text",
        )
        assert_matches_type(BroadcastUpdateResponse, broadcast, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_update(self, async_client: AsyncZavudev) -> None:
        response = await async_client.broadcasts.with_raw_response.update(
            broadcast_id="broadcastId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        broadcast = await response.parse()
        assert_matches_type(BroadcastUpdateResponse, broadcast, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncZavudev) -> None:
        async with async_client.broadcasts.with_streaming_response.update(
            broadcast_id="broadcastId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            broadcast = await response.parse()
            assert_matches_type(BroadcastUpdateResponse, broadcast, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_update(self, async_client: AsyncZavudev) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `broadcast_id` but received ''"):
            await async_client.broadcasts.with_raw_response.update(
                broadcast_id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncZavudev) -> None:
        broadcast = await async_client.broadcasts.list()
        assert_matches_type(AsyncCursor[Broadcast], broadcast, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncZavudev) -> None:
        broadcast = await async_client.broadcasts.list(
            cursor="cursor",
            limit=100,
            status="draft",
        )
        assert_matches_type(AsyncCursor[Broadcast], broadcast, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncZavudev) -> None:
        response = await async_client.broadcasts.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        broadcast = await response.parse()
        assert_matches_type(AsyncCursor[Broadcast], broadcast, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncZavudev) -> None:
        async with async_client.broadcasts.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            broadcast = await response.parse()
            assert_matches_type(AsyncCursor[Broadcast], broadcast, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_delete(self, async_client: AsyncZavudev) -> None:
        broadcast = await async_client.broadcasts.delete(
            "broadcastId",
        )
        assert broadcast is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncZavudev) -> None:
        response = await async_client.broadcasts.with_raw_response.delete(
            "broadcastId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        broadcast = await response.parse()
        assert broadcast is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncZavudev) -> None:
        async with async_client.broadcasts.with_streaming_response.delete(
            "broadcastId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            broadcast = await response.parse()
            assert broadcast is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_delete(self, async_client: AsyncZavudev) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `broadcast_id` but received ''"):
            await async_client.broadcasts.with_raw_response.delete(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_cancel(self, async_client: AsyncZavudev) -> None:
        broadcast = await async_client.broadcasts.cancel(
            "broadcastId",
        )
        assert_matches_type(BroadcastCancelResponse, broadcast, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_cancel(self, async_client: AsyncZavudev) -> None:
        response = await async_client.broadcasts.with_raw_response.cancel(
            "broadcastId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        broadcast = await response.parse()
        assert_matches_type(BroadcastCancelResponse, broadcast, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_cancel(self, async_client: AsyncZavudev) -> None:
        async with async_client.broadcasts.with_streaming_response.cancel(
            "broadcastId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            broadcast = await response.parse()
            assert_matches_type(BroadcastCancelResponse, broadcast, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_cancel(self, async_client: AsyncZavudev) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `broadcast_id` but received ''"):
            await async_client.broadcasts.with_raw_response.cancel(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_progress(self, async_client: AsyncZavudev) -> None:
        broadcast = await async_client.broadcasts.progress(
            "broadcastId",
        )
        assert_matches_type(BroadcastProgress, broadcast, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_progress(self, async_client: AsyncZavudev) -> None:
        response = await async_client.broadcasts.with_raw_response.progress(
            "broadcastId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        broadcast = await response.parse()
        assert_matches_type(BroadcastProgress, broadcast, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_progress(self, async_client: AsyncZavudev) -> None:
        async with async_client.broadcasts.with_streaming_response.progress(
            "broadcastId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            broadcast = await response.parse()
            assert_matches_type(BroadcastProgress, broadcast, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_progress(self, async_client: AsyncZavudev) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `broadcast_id` but received ''"):
            await async_client.broadcasts.with_raw_response.progress(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_reschedule(self, async_client: AsyncZavudev) -> None:
        broadcast = await async_client.broadcasts.reschedule(
            broadcast_id="broadcastId",
            scheduled_at=parse_datetime("2024-01-15T14:00:00Z"),
        )
        assert_matches_type(BroadcastRescheduleResponse, broadcast, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_reschedule(self, async_client: AsyncZavudev) -> None:
        response = await async_client.broadcasts.with_raw_response.reschedule(
            broadcast_id="broadcastId",
            scheduled_at=parse_datetime("2024-01-15T14:00:00Z"),
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        broadcast = await response.parse()
        assert_matches_type(BroadcastRescheduleResponse, broadcast, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_reschedule(self, async_client: AsyncZavudev) -> None:
        async with async_client.broadcasts.with_streaming_response.reschedule(
            broadcast_id="broadcastId",
            scheduled_at=parse_datetime("2024-01-15T14:00:00Z"),
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            broadcast = await response.parse()
            assert_matches_type(BroadcastRescheduleResponse, broadcast, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_reschedule(self, async_client: AsyncZavudev) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `broadcast_id` but received ''"):
            await async_client.broadcasts.with_raw_response.reschedule(
                broadcast_id="",
                scheduled_at=parse_datetime("2024-01-15T14:00:00Z"),
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_send(self, async_client: AsyncZavudev) -> None:
        broadcast = await async_client.broadcasts.send(
            broadcast_id="broadcastId",
        )
        assert_matches_type(BroadcastSendResponse, broadcast, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_send_with_all_params(self, async_client: AsyncZavudev) -> None:
        broadcast = await async_client.broadcasts.send(
            broadcast_id="broadcastId",
            scheduled_at=parse_datetime("2019-12-27T18:11:19.117Z"),
        )
        assert_matches_type(BroadcastSendResponse, broadcast, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_send(self, async_client: AsyncZavudev) -> None:
        response = await async_client.broadcasts.with_raw_response.send(
            broadcast_id="broadcastId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        broadcast = await response.parse()
        assert_matches_type(BroadcastSendResponse, broadcast, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_send(self, async_client: AsyncZavudev) -> None:
        async with async_client.broadcasts.with_streaming_response.send(
            broadcast_id="broadcastId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            broadcast = await response.parse()
            assert_matches_type(BroadcastSendResponse, broadcast, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_send(self, async_client: AsyncZavudev) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `broadcast_id` but received ''"):
            await async_client.broadcasts.with_raw_response.send(
                broadcast_id="",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from zavudev import Zavudev, AsyncZavudev
from tests.utils import assert_matches_type
from zavudev.types import (
    RegulatoryDocument,
    RegulatoryDocumentCreateResponse,
    RegulatoryDocumentRetrieveResponse,
    RegulatoryDocumentUploadURLResponse,
)
from zavudev.pagination import SyncCursor, AsyncCursor

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestRegulatoryDocuments:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create(self, client: Zavudev) -> None:
        regulatory_document = client.regulatory_documents.create(
            document_type="passport",
            file_size=102400,
            mime_type="image/jpeg",
            name="Passport Scan",
            storage_id="kg2abc123...",
        )
        assert_matches_type(RegulatoryDocumentCreateResponse, regulatory_document, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_create(self, client: Zavudev) -> None:
        response = client.regulatory_documents.with_raw_response.create(
            document_type="passport",
            file_size=102400,
            mime_type="image/jpeg",
            name="Passport Scan",
            storage_id="kg2abc123...",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        regulatory_document = response.parse()
        assert_matches_type(RegulatoryDocumentCreateResponse, regulatory_document, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_create(self, client: Zavudev) -> None:
        with client.regulatory_documents.with_streaming_response.create(
            document_type="passport",
            file_size=102400,
            mime_type="image/jpeg",
            name="Passport Scan",
            storage_id="kg2abc123...",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            regulatory_document = response.parse()
            assert_matches_type(RegulatoryDocumentCreateResponse, regulatory_document, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Zavudev) -> None:
        regulatory_document = client.regulatory_documents.retrieve(
            "documentId",
        )
        assert_matches_type(RegulatoryDocumentRetrieveResponse, regulatory_document, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Zavudev) -> None:
        response = client.regulatory_documents.with_raw_response.retrieve(
            "documentId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        regulatory_document = response.parse()
        assert_matches_type(RegulatoryDocumentRetrieveResponse, regulatory_document, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Zavudev) -> None:
        with client.regulatory_documents.with_streaming_response.retrieve(
            "documentId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            regulatory_document = response.parse()
            assert_matches_type(RegulatoryDocumentRetrieveResponse, regulatory_document, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Zavudev) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `document_id` but received ''"):
            client.regulatory_documents.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list(self, client: Zavudev) -> None:
        regulatory_document = client.regulatory_documents.list()
        assert_matches_type(SyncCursor[RegulatoryDocument], regulatory_document, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: Zavudev) -> None:
        regulatory_document = client.regulatory_documents.list(
            cursor="cursor",
            limit=100,
        )
        assert_matches_type(SyncCursor[RegulatoryDocument], regulatory_document, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Zavudev) -> None:
        response = client.regulatory_documents.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        regulatory_document = response.parse()
        assert_matches_type(SyncCursor[RegulatoryDocument], regulatory_document, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Zavudev) -> None:
        with client.regulatory_documents.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            regulatory_document = response.parse()
            assert_matches_type(SyncCursor[RegulatoryDocument], regulatory_document, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_delete(self, client: Zavudev) -> None:
        regulatory_document = client.regulatory_documents.delete(
            "documentId",
        )
        assert regulatory_document is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_delete(self, client: Zavudev) -> None:
        response = client.regulatory_documents.with_raw_response.delete(
            "documentId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        regulatory_document = response.parse()
        assert regulatory_document is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_delete(self, client: Zavudev) -> None:
        with client.regulatory_documents.with_streaming_response.delete(
            "documentId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            regulatory_document = response.parse()
            assert regulatory_document is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_delete(self, client: Zavudev) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `document_id` but received ''"):
            client.regulatory_documents.with_raw_response.delete(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_upload_url(self, client: Zavudev) -> None:
        regulatory_document = client.regulatory_documents.upload_url()
        assert_matches_type(RegulatoryDocumentUploadURLResponse, regulatory_document, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_upload_url(self, client: Zavudev) -> None:
        response = client.regulatory_documents.with_raw_response.upload_url()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        regulatory_document = response.parse()
        assert_matches_type(RegulatoryDocumentUploadURLResponse, regulatory_document, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_upload_url(self, client: Zavudev) -> None:
        with client.regulatory_documents.with_streaming_response.upload_url() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            regulatory_document = response.parse()
            assert_matches_type(RegulatoryDocumentUploadURLResponse, regulatory_document, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncRegulatoryDocuments:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create(self, async_client: AsyncZavudev) -> None:
        regulatory_document = await async_client.regulatory_documents.create(
            document_type="passport",
            file_size=102400,
            mime_type="image/jpeg",
            name="Passport Scan",
            storage_id="kg2abc123...",
        )
        assert_matches_type(RegulatoryDocumentCreateResponse, regulatory_document, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncZavudev) -> None:
        response = await async_client.regulatory_documents.with_raw_response.create(
            document_type="passport",
            file_size=102400,
            mime_type="image/jpeg",
            name="Passport Scan",
            storage_id="kg2abc123...",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        regulatory_document = await response.parse()
        assert_matches_type(RegulatoryDocumentCreateResponse, regulatory_document, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncZavudev) -> None:
        async with async_client.regulatory_documents.with_streaming_response.create(
            document_type="passport",
            file_size=102400,
            mime_type="image/jpeg",
            name="Passport Scan",
            storage_id="kg2abc123...",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            regulatory_document = await response.parse()
            assert_matches_type(RegulatoryDocumentCreateResponse, regulatory_document, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncZavudev) -> None:
        regulatory_document = await async_client.regulatory_documents.retrieve(
            "documentId",
        )
        assert_matches_type(RegulatoryDocumentRetrieveResponse, regulatory_document, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncZavudev) -> None:
        response = await async_client.regulatory_documents.with_raw_response.retrieve(
            "documentId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        regulatory_document = await response.parse()
        assert_matches_type(RegulatoryDocumentRetrieveResponse, regulatory_document, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncZavudev) -> None:
        async with async_client.regulatory_documents.with_streaming_response.retrieve(
            "documentId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            regulatory_document = await response.parse()
            assert_matches_type(RegulatoryDocumentRetrieveResponse, regulatory_document, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncZavudev) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `document_id` but received ''"):
            await async_client.regulatory_documents.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncZavudev) -> None:
        regulatory_document = await async_client.regulatory_documents.list()
        assert_matches_type(AsyncCursor[RegulatoryDocument], regulatory_document, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncZavudev) -> None:
        regulatory_document = await async_client.regulatory_documents.list(
            cursor="cursor",
            limit=100,
        )
        assert_matches_type(AsyncCursor[RegulatoryDocument], regulatory_document, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncZavudev) -> None:
        response = await async_client.regulatory_documents.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        regulatory_document = await response.parse()
        assert_matches_type(AsyncCursor[RegulatoryDocument], regulatory_document, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncZavudev) -> None:
        async with async_client.regulatory_documents.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            regulatory_document = await response.parse()
            assert_matches_type(AsyncCursor[RegulatoryDocument], regulatory_document, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_delete(self, async_client: AsyncZavudev) -> None:
        regulatory_document = await async_client.regulatory_documents.delete(
            "documentId",
        )
        assert regulatory_document is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncZavudev) -> None:
        response = await async_client.regulatory_documents.with_raw_response.delete(
            "documentId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        regulatory_document = await response.parse()
        assert regulatory_document is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncZavudev) -> None:
        async with async_client.regulatory_documents.with_streaming_response.delete(
            "documentId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            regulatory_document = await response.parse()
            assert regulatory_document is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_delete(self, async_client: AsyncZavudev) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `document_id` but received ''"):
            await async_client.regulatory_documents.with_raw_response.delete(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_upload_url(self, async_client: AsyncZavudev) -> None:
        regulatory_document = await async_client.regulatory_documents.upload_url()
        assert_matches_type(RegulatoryDocumentUploadURLResponse, regulatory_document, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_upload_url(self, async_client: AsyncZavudev) -> None:
        response = await async_client.regulatory_documents.with_raw_response.upload_url()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        regulatory_document = await response.parse()
        assert_matches_type(RegulatoryDocumentUploadURLResponse, regulatory_document, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_upload_url(self, async_client: AsyncZavudev) -> None:
        async with async_client.regulatory_documents.with_streaming_response.upload_url() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            regulatory_document = await response.parse()
            assert_matches_type(RegulatoryDocumentUploadURLResponse, regulatory_document, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Iterable
from typing_extensions import Required, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["ContactAddParams", "Contact"]


class ContactAddParams(TypedDict, total=False):
    contacts: Required[Iterable[Contact]]
    """List of contacts to add (max 1000 per request)."""


class Contact(TypedDict, total=False):
    recipient: Required[str]
    """Phone number (E.164) or email address."""

    template_variables: Annotated[Dict[str, str], PropertyInfo(alias="templateVariables")]
    """Per-contact template variables to personalize the message."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from zavudev import Zavudev, AsyncZavudev
from tests.utils import assert_matches_type
from zavudev.pagination import SyncCursor, AsyncCursor
from zavudev.types.senders.agent import (
    AgentFlow,
    FlowCreateResponse,
    FlowUpdateResponse,
    FlowRetrieveResponse,
    FlowDuplicateResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestFlows:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create(self, client: Zavudev) -> None:
        flow = client.senders.agent.flows.create(
            sender_id="senderId",
            name="Lead Capture",
            steps=[
                {
                    "id": "welcome",
                    "config": {"text": "bar"},
                    "type": "message",
                },
                {
                    "id": "ask_name",
                    "config": {
                        "variable": "bar",
                        "prompt": "bar",
                    },
                    "type": "collect",
                },
            ],
            trigger={"type": "keyword"},
        )
        assert_matches_type(FlowCreateResponse, flow, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create_with_all_params(self, client: Zavudev) -> None:
        flow = client.senders.agent.flows.create(
            sender_id="senderId",
            name="Lead Capture",
            steps=[
                {
                    "id": "welcome",
                    "config": {"text": "bar"},
                    "type": "message",
                    "next_step_id": "ask_name",
                },
                {
                    "id": "ask_name",
                    "config": {
                        "variable": "bar",
                        "prompt": "bar",
                    },
                    "type": "collect",
                    "next_step_id": "nextStepId",
                },
            ],
            trigger={
                "type": "keyword",
                "intent": "intent",
                "keywords": ["info", "pricing", "demo"],
            },
            description="Capture lead information",
            enabled=True,
            priority=0,
        )
        assert_matches_type(FlowCreateResponse, flow, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_create(self, client: Zavudev) -> None:
        response = client.senders.agent.flows.with_raw_response.create(
            sender_id="senderId",
            name="Lead Capture",
            steps=[
                {
                    "id": "welcome",
                    "config": {"text": "bar"},
                    "type": "message",
                },
                {
                    "id": "ask_name",
                    "config": {
                        "variable": "bar",
                        "prompt": "bar",
                    },
                    "type": "collect",
                },
            ],
            trigger={"type": "keyword"},
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        flow = response.parse()
        assert_matches_type(FlowCreateResponse, flow, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_create(self, client: Zavudev) -> None:
        with client.senders.agent.flows.with_streaming_response.create(
            sender_id="senderId",
            name="Lead Capture",
            steps=[
                {
                    "id": "welcome",
                    "config": {"text": "bar"},
                    "type": "message",
                },
                {
                    "id": "ask_name",
                    "config": {
                        "variable": "bar",
                        "prompt": "bar",
                    },
                    "type": "collect",
                },
            ],
            trigger={"type": "keyword"},
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            flow = response.parse()
            assert_matches_type(FlowCreateResponse, flow, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_create(self, client: Zavudev) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `sender_id` but received ''"):
            client.senders.agent.flows.with_raw_response.create(
                sender_id="",
                name="Lead Capture",
                steps=[
                    {
                        "id": "welcome",
                        "config": {"text": "bar"},
                        "type": "message",
                    },
                    {
                        "id": "ask_name",
                        "config": {
                            "variable": "bar",
                            "prompt": "bar",
                        },
                        "type": "collect",
                    },
                ],
                trigger={"type": "keyword"},
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Zavudev) -> None:
        flow = client.senders.agent.flows.retrieve(
            flow_id="flowId",
            sender_id="senderId",
        )
        assert_matches_type(FlowRetrieveResponse, flow, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Zavudev) -> None:
        response = client.senders.agent.flows.with_raw_response.retrieve(
            flow_id="flowId",
            sender_id="senderId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        flow = response.parse()
        assert_matches_type(FlowRetrieveResponse, flow, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Zavudev) -> None:
        with client.senders.agent.flows.with_streaming_response.retrieve(
            flow_id="flowId",
            sender_id="senderId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            flow = response.parse()
            assert_matches_type(FlowRetrieveResponse, flow, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Zavudev) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `sender_id` but received ''"):
            client.senders.agent.flows.with_raw_response.retrieve(
                flow_id="flowId",
                sender_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `flow_id` but received ''"):
            client.senders.agent.flows.with_raw_response.retrieve(
                flow_id="",
                sender_id="senderId",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_update(self, client: Zavudev) -> None:
        flow = client.senders.agent.flows.update(
            flow_id="flowId",
            sender_id="senderId",
        )
        assert_matches_type(FlowUpdateResponse, flow, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_update_with_all_params(self, client: Zavudev) -> None:
        flow = client.senders.agent.flows.update(
            flow_id="flowId",
            sender_id="senderId",
            description="description",
            enabled=True,
            name="name",
            priority=0,
            steps=[
                {
                    "id": "id",
                    "config": {"foo": "bar"},
                    "type": "message",
                    "next_step_id": "nextStepId",
                }
            ],
            trigger={
                "type": "keyword",
                "intent": "intent",
                "keywords": ["string"],
            },
        )
        assert_matches_type(FlowUpdateResponse, flow, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_update(self, client: Zavudev) -> None:
        response = client.senders.agent.flows.with_raw_response.update(
            flow_id="flowId",
            sender_id="senderId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        flow = response.parse()
        assert_matches_type(FlowUpdateResponse, flow, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_update(self, client: Zavudev) -> None:
        with client.senders.agent.flows.with_streaming_response.update(
            flow_id="flowId",
            sender_id="senderId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            flow = response.parse()
            assert_matches_type(FlowUpdateResponse, flow, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_update(self, client: Zavudev) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `sender_id` but received ''"):
            client.senders.agent.flows.with_raw_response.update(
                flow_id="flowId",
                sender_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `flow_id` but received ''"):
            client.senders.agent.flows.with_raw_response.update(
                flow_id="",
                sender_id="senderId",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list(self, client: Zavudev) -> None:
        flow = client.senders.agent.flows.list(
            sender_id="senderId",
        )
        assert_matches_type(SyncCursor[AgentFlow], flow, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: Zavudev) -> None:
        flow = client.senders.agent.flows.list(
            sender_id="senderId",
            cursor="cursor",
            enabled=True,
            limit=100,
        )
        assert_matches_type(SyncCursor[AgentFlow], flow, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Zavudev) -> None:
        response = client.senders.agent.flows.with_raw_response.list(
            sender_id="senderId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        flow = response.parse()
        assert_matches_type(SyncCursor[AgentFlow], flow, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Zavudev) -> None:
        with client.senders.agent.flows.with_streaming_response.list(
            sender_id="senderId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            flow = response.parse()
            assert_matches_type(SyncCursor[AgentFlow], flow, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_list(self, client: Zavudev) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `sender_id` but received ''"):
            client.senders.agent.flows.with_raw_response.list(
                sender_id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_delete(self, client: Zavudev) -> None:
        flow = client.senders.agent.flows.delete(
            flow_id="flowId",
            sender_id="senderId",
        )
        assert flow is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_delete(self, client: Zavudev) -> None:
        response = client.senders.agent.flows.with_raw_response.delete(
            flow_id="flowId",
            sender_id="senderId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        flow = response.parse()
        assert flow is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_delete(self, client: Zavudev) -> None:
        with client.senders.agent.flows.with_streaming_response.delete(
            flow_id="flowId",
            sender_id="senderId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            flow = response.parse()
            assert flow is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_delete(self, client: Zavudev) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `sender_id` but received ''"):
            client.senders.agent.flows.with_raw_response.delete(
                flow_id="flowId",
                sender_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `flow_id` but received ''"):
            client.senders.agent.flows.with_raw_response.delete(
                flow_id="",
                sender_id="senderId",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_duplicate(self, client: Zavudev) -> None:
        flow = client.senders.agent.flows.duplicate(
            flow_id="flowId",
            sender_id="senderId",
            new_name="Lead Capture (Copy)",
        )
        assert_matches_type(FlowDuplicateResponse, flow, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_duplicate(self, client: Zavudev) -> None:
        response = client.senders.agent.flows.with_raw_response.duplicate(
            flow_id="flowId",
            sender_id="senderId",
            new_name="Lead Capture (Copy)",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        flow = response.parse()
        assert_matches_type(FlowDuplicateResponse, flow, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_duplicate(self, client: Zavudev) -> None:
        with client.senders.agent.flows.with_streaming_response.duplicate(
            flow_id="flowId",
            sender_id="senderId",
            new_name="Lead Capture (Copy)",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            flow = response.parse()
            assert_matches_type(FlowDuplicateResponse, flow, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_duplicate(self, client: Zavudev) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `sender_id` but received ''"):
            client.senders.agent.flows.with_raw_response.duplicate(
                flow_id="flowId",
                sender_id="",
                new_name="Lead Capture (Copy)",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `flow_id` but received ''"):
            client.senders.agent.flows.with_raw_response.duplicate(
                flow_id="",
                sender_id="senderId",
                new_name="Lead Capture (Copy)",
            )


class TestAsyncFlows:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create(self, async_client: AsyncZavudev) -> None:
        flow = await async_client.senders.agent.flows.create(
            sender_id="senderId",
            name="Lead Capture",
            steps=[
                {
                    "id": "welcome",
                    "config": {"text": "bar"},
                    "type": "message",
                },
                {
                    "id": "ask_name",
                    "config": {
                        "variable": "bar",
                        "prompt": "bar",
                    },
                    "type": "collect",
                },
            ],
            trigger={"type": "keyword"},
        )
        assert_matches_type(FlowCreateResponse, flow, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncZavudev) -> None:
        flow = await async_client.senders.agent.flows.create(
            sender_id="senderId",
            name="Lead Capture",
            steps=[
                {
                    "id": "welcome",
                    "config": {"text": "bar"},
                    "type": "message",
                    "next_step_id": "ask_name",
                },
                {
                    "id": "ask_name",
                    "config": {
                        "variable": "bar",
                        "prompt": "bar",
                    },
                    "type": "collect",
                    "next_step_id": "nextStepId",
                },
            ],
            trigger={
                "type": "keyword",
                "intent": "intent",
                "keywords": ["info", "pricing", "demo"],
            },
            description="Capture lead information",
            enabled=True,
            priority=0,
        )
        assert_matches_type(FlowCreateResponse, flow, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncZavudev) -> None:
        response = await async_client.senders.agent.flows.with_raw_response.create(
            sender_id="senderId",
            name="Lead Capture",
            steps=[
                {
                    "id": "welcome",
                    "config": {"text": "bar"},
                    "type": "message",
                },
                {
                    "id": "ask_name",
                    "config": {
                        "variable": "bar",
                        "prompt": "bar",
                    },
                    "type": "collect",
                },
            ],
            trigger={"type": "keyword"},
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        flow = await response.parse()
        assert_matches_type(FlowCreateResponse, flow, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncZavudev) -> None:
        async with async_client.senders.agent.flows.with_streaming_response.create(
            sender_id="senderId",
            name="Lead Capture",
            steps=[
                {
                    "id": "welcome",
                    "config": {"text": "bar"},
                    "type": "message",
                },
                {
                    "id": "ask_name",
                    "config": {
                        "variable": "bar",
                        "prompt": "bar",
                    },
                    "type": "collect",
                },
            ],
            trigger={"type": "keyword"},
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            flow = await response.parse()
            assert_matches_type(FlowCreateResponse, flow, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_create(self, async_client: AsyncZavudev) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `sender_id` but received ''"):
            await async_client.senders.agent.flows.with_raw_response.create(
                sender_id="",
                name="Lead Capture",
                steps=[
                    {
                        "id": "welcome",
                        "config": {"text": "bar"},
                        "type": "message",
                    },
                    {
                        "id": "ask_name",
                        "config": {
                            "variable": "bar",
                            "prompt": "bar",
                        },
                        "type": "collect",
                    },
                ],
                trigger={"type": "keyword"},
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncZavudev) -> None:
        flow = await async_client.senders.agent.flows.retrieve(
            flow_id="flowId",
            sender_id="senderId",
        )
        assert_matches_type(FlowRetrieveResponse, flow, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncZavudev) -> None:
        response = await async_client.senders.agent.flows.with_raw_response.retrieve(
            flow_id="flowId",
            sender_id="senderId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        flow = await response.parse()
        assert_matches_type(FlowRetrieveResponse, flow, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncZavudev) -> None:
        async with async_client.senders.agent.flows.with_streaming_response.retrieve(
            flow_id="flowId",
            sender_id="senderId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            flow = await response.parse()
            assert_matches_type(FlowRetrieveResponse, flow, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncZavudev) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `sender_id` but received ''"):
            await async_client.senders.agent.flows.with_raw_response.retrieve(
                flow_id="flowId",
                sender_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `flow_id` but received ''"):
            await async_client.senders.agent.flows.with_raw_response.retrieve(
                flow_id="",
                sender_id="senderId",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_update(self, async_client: AsyncZavudev) -> None:
        flow = await async_client.senders.agent.flows.update(
            flow_id="flowId",
            sender_id="senderId",
        )
        assert_matches_type(FlowUpdateResponse, flow, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncZavudev) -> None:
        flow = await async_client.senders.agent.flows.update(
            flow_id="flowId",
            sender_id="senderId",
            description="description",
            enabled=True,
            name="name",
            priority=0,
            steps=[
                {
                    "id": "id",
                    "config": {"foo": "bar"},
                    "type": "message",
                    "next_step_id": "nextStepId",
                }
            ],
            trigger={
                "type": "keyword",
                "intent": "intent",
                "keywords": ["string"],
            },
        )
        assert_matches_type(FlowUpdateResponse, flow, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_update(self, async_client: AsyncZavudev) -> None:
        response = await async_client.senders.agent.flows.with_raw_response.update(
            flow_id="flowId",
            sender_id="senderId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        flow = await response.parse()
        assert_matches_type(FlowUpdateResponse, flow, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncZavudev) -> None:
        async with async_client.senders.agent.flows.with_streaming_response.update(
            flow_id="flowId",
            sender_id="senderId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            flow = await response.parse()
            assert_matches_type(FlowUpdateResponse, flow, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_update(self, async_client: AsyncZavudev) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `sender_id` but received ''"):
            await async_client.senders.agent.flows.with_raw_response.update(
                flow_id="flowId",
                sender_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `flow_id` but received ''"):
            await async_client.senders.agent.flows.with_raw_response.update(
                flow_id="",
                sender_id="senderId",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncZavudev) -> None:
        flow = await async_client.senders.agent.flows.list(
            sender_id="senderId",
        )
        assert_matches_type(AsyncCursor[AgentFlow], flow, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncZavudev) -> None:
        flow = await async_client.senders.agent.flows.list(
            sender_id="senderId",
            cursor="cursor",
            enabled=True,
            limit=100,
        )
        assert_matches_type(AsyncCursor[AgentFlow], flow, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncZavudev) -> None:
        response = await async_client.senders.agent.flows.with_raw_response.list(
            sender_id="senderId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        flow = await response.parse()
        assert_matches_type(AsyncCursor[AgentFlow], flow, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncZavudev) -> None:
        async with async_client.senders.agent.flows.with_streaming_response.list(
            sender_id="senderId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            flow = await response.parse()
            assert_matches_type(AsyncCursor[AgentFlow], flow, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_list(self, async_client: AsyncZavudev) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `sender_id` but received ''"):
            await async_client.senders.agent.flows.with_raw_response.list(
                sender_id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_delete(self, async_client: AsyncZavudev) -> None:
        flow = await async_client.senders.agent.flows.delete(
            flow_id="flowId",
            sender_id="senderId",
        )
        assert flow is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncZavudev) -> None:
        response = await async_client.senders.agent.flows.with_raw_response.delete(
            flow_id="flowId",
            sender_id="senderId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        flow = await response.parse()
        assert flow is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncZavudev) -> None:
        async with async_client.senders.agent.flows.with_streaming_response.delete(
            flow_id="flowId",
            sender_id="senderId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            flow = await response.parse()
            assert flow is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_delete(self, async_client: AsyncZavudev) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `sender_id` but received ''"):
            await async_client.senders.agent.flows.with_raw_response.delete(
                flow_id="flowId",
                sender_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `flow_id` but received ''"):
            await async_client.senders.agent.flows.with_raw_response.delete(
                flow_id="",
                sender_id="senderId",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_duplicate(self, async_client: AsyncZavudev) -> None:
        flow = await async_client.senders.agent.flows.duplicate(
            flow_id="flowId",
            sender_id="senderId",
            new_name="Lead Capture (Copy)",
        )
        assert_matches_type(FlowDuplicateResponse, flow, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_duplicate(self, async_client: AsyncZavudev) -> None:
        response = await async_client.senders.agent.flows.with_raw_response.duplicate(
            flow_id="flowId",
            sender_id="senderId",
            new_name="Lead Capture (Copy)",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        flow = await response.parse()
        assert_matches_type(FlowDuplicateResponse, flow, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_duplicate(self, async_client: AsyncZavudev) -> None:
        async with async_client.senders.agent.flows.with_streaming_response.duplicate(
            flow_id="flowId",
            sender_id="senderId",
            new_name="Lead Capture (Copy)",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            flow = await response.parse()
            assert_matches_type(FlowDuplicateResponse, flow, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_duplicate(self, async_client: AsyncZavudev) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `sender_id` but received ''"):
            await async_client.senders.agent.flows.with_raw_response.duplicate(
                flow_id="flowId",
                sender_id="",
                new_name="Lead Capture (Copy)",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `flow_id` but received ''"):
            await async_client.senders.agent.flows.with_raw_response.duplicate(
                flow_id="",
                sender_id="senderId",
                new_name="Lead Capture (Copy)",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["ContactListParams"]


class ContactListParams(TypedDict, total=False):
    cursor: str

    limit: int

    phone_number: Annotated[str, PropertyInfo(alias="phoneNumber")]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from pydantic import Field as FieldInfo

from .._models import BaseModel
from .owned_phone_number import OwnedPhoneNumber

__all__ = ["PhoneNumberPurchaseResponse"]


class PhoneNumberPurchaseResponse(BaseModel):
    phone_number: OwnedPhoneNumber = FieldInfo(alias="phoneNumber")



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from ...._models import BaseModel

__all__ = ["ToolTestResponse"]


class ToolTestResponse(BaseModel):
    scheduled: bool
